Package: ORFik
Type: Package
Title: Open Reading Frames in Genomics
Version: 1.30.0
Encoding: UTF-8
Authors@R: c(
  person(given = "Haakon", family = "Tjeldnes", email = "hauken_heyken@hotmail.com", role = c("aut", "cre", "dtc")),
  person(given = "Kornel", family = "Labun", email = "kornellabun@gmail.com", role = c("aut", "cph")),
  person(given = "Michal", family = "Swirski", email = "michal.swirski@uw.edu.pl", role = c("ctb")),
  person(given = "Katarzyna", family = "Chyzynska", email = "katchyz@gmail.com", role = c("ctb", "dtc")),
  person(given = "Yamila", family = "Torres Cleuren", email = "yamilatorrescleuren@gmail.com", role = c("ctb", "ths")),
  person(given = "Eivind", family = "Valen", email = "eivind.valen@gmail.com", role = c("ths", "fnd")))
Description: R package for analysis of transcript and translation features through manipulation of sequence data
  and NGS data like Ribo-Seq, RNA-Seq, TCP-Seq and CAGE. It is generalized in the sense that any transcript region
  can be analysed, as the name hints to it was made with investigation of ribosomal patterns over 
  Open Reading Frames (ORFs) as it's primary use case. 
  ORFik is extremely fast through use of C++, data.table and GenomicRanges. 
  Package allows to reassign starts of the transcripts with the use of CAGE-Seq data,
  automatic shifting of RiboSeq reads, finding of Open Reading Frames for
  whole genomes and much more.
biocViews: ImmunoOncology, Software, Sequencing, RiboSeq, RNASeq,
        FunctionalGenomics, Coverage, Alignment, DataImport
License: MIT + file LICENSE
BugReports: https://github.com/Roleren/ORFik/issues
URL: https://github.com/Roleren/ORFik
Depends: R (>= 4.1.0), IRanges (>= 2.17.1), GenomicRanges (>= 1.35.1),
        GenomicAlignments (>= 1.19.0)
Imports: AnnotationDbi (>= 1.45.0), Biostrings (>= 2.51.1), biomaRt,
        biomartr (>= 1.0.7), BiocFileCache, BiocGenerics (>= 0.29.1),
        BiocParallel (>= 1.19.0), BSgenome, cowplot (>= 1.0.0),
        data.table (>= 1.11.8), DESeq2 (>= 1.24.0), fst (>= 0.9.2),
        GenomeInfoDb (>= 1.15.5), GenomicFeatures (>= 1.31.10), ggplot2
        (>= 2.2.1), gridExtra (>= 2.3), httr (>= 1.3.0), jsonlite,
        methods (>= 3.6.0), qs, R.utils, Rcpp (>= 1.0.0), Rsamtools (>=
        1.35.0), rtracklayer (>= 1.43.0), stats, SummarizedExperiment
        (>= 1.14.0), S4Vectors (>= 0.21.3), tools, txdbmaker, utils,
        XML, xml2 (>= 1.2.0), withr
RoxygenNote: 7.3.1
Suggests: testthat, rmarkdown, knitr, BiocStyle,
        BSgenome.Hsapiens.UCSC.hg19, GenomeInfoDbData
LinkingTo: Rcpp
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/ORFik
git_branch: RELEASE_3_22
git_last_commit: b860267
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 05:37:41 UTC; biocbuild
Author: Haakon Tjeldnes [aut, cre, dtc],
  Kornel Labun [aut, cph],
  Michal Swirski [ctb],
  Katarzyna Chyzynska [ctb, dtc],
  Yamila Torres Cleuren [ctb, ths],
  Eivind Valen [ths, fnd]
Maintainer: Haakon Tjeldnes <hauken_heyken@hotmail.com>
