# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

orfs_as_IRanges <- function(main_string, s, e, minimumLength) {
    .Call('_ORFik_orfs_as_IRanges', PACKAGE = 'ORFik', main_string, s, e, minimumLength)
}

orfs_as_List <- function(fastaSeqs, startCodon, stopCodon, minimumLength) {
    .Call('_ORFik_orfs_as_List', PACKAGE = 'ORFik', fastaSeqs, startCodon, stopCodon, minimumLength)
}

findORFs_fasta <- function(fastaSeqs, startCodon, stopCodon, minimumLength, isCircular) {
    .Call('_ORFik_findORFs_fasta', PACKAGE = 'ORFik', fastaSeqs, startCodon, stopCodon, minimumLength, isCircular)
}

pmapFromTranscriptsCPP <- function(xStart, xEnd, transcriptStart, transcriptEnd, indices, direction, removeEmpty) {
    .Call('_ORFik_pmapFromTranscriptsCPP', PACKAGE = 'ORFik', xStart, xEnd, transcriptStart, transcriptEnd, indices, direction, removeEmpty)
}

pmapToTranscriptsCPP <- function(xStart, xEnd, transcriptStart, transcriptEnd, indices, direction, nExonCumSum) {
    .Call('_ORFik_pmapToTranscriptsCPP', PACKAGE = 'ORFik', xStart, xEnd, transcriptStart, transcriptEnd, indices, direction, nExonCumSum)
}

