% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validity.R
\name{checkRFP}
\alias{checkRFP}
\title{Helper Function to check valid RFP input}
\usage{
checkRFP(class)
}
\arguments{
\item{class, }{the given class of RFP object}
}
\value{
NULL, stop if invalid object
}
\description{
Helper Function to check valid RFP input
}
\seealso{
Other validity: 
\code{\link{checkRNA}()},
\code{\link{is.ORF}()},
\code{\link{is.gr_or_grl}()},
\code{\link{is.grl}()},
\code{\link{is.range}()},
\code{\link{validGRL}()},
\code{\link{validSeqlevels}()}
}
\concept{validity}
\keyword{internal}
