% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uORF_helpers.R
\name{removeORFsWithSameStopAsCDS}
\alias{removeORFsWithSameStopAsCDS}
\title{Remove ORFs that have same stop site as the CDS}
\usage{
removeORFsWithSameStopAsCDS(grl, cds)
}
\arguments{
\item{grl}{(GRangesList), the ORFs to filter}

\item{cds}{(GRangesList), the coding sequences (main ORFs on transcripts),
to filter against.}
}
\value{
(GRangesList) of filtered uORFs
}
\description{
Remove ORFs that have same stop site as the CDS
}
\seealso{
Other uorfs: 
\code{\link{addCdsOnLeaderEnds}()},
\code{\link{filterUORFs}()},
\code{\link{removeORFsWithSameStartAsCDS}()},
\code{\link{removeORFsWithStartInsideCDS}()},
\code{\link{removeORFsWithinCDS}()},
\code{\link{uORFSearchSpace}()}
}
\concept{uorfs}
\keyword{internal}
