% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/experiment_class.R
\name{runIDs}
\alias{runIDs}
\title{Get SRR/DRR/ERR run ids from ORFik experiment}
\usage{
runIDs(x)
}
\arguments{
\item{x}{an ORFik \code{\link{experiment}}}
}
\value{
a character vector of runIDs, "" if not existing.
}
\description{
Get SRR/DRR/ERR run ids from ORFik experiment
}
