% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cage_annotations.R
\name{findMaxPeaks}
\alias{findMaxPeaks}
\title{Find max peak for each transcript,
returns as data.table, without names, but with index}
\usage{
findMaxPeaks(cageOverlaps, filteredCage)
}
\arguments{
\item{cageOverlaps}{The cageOverlaps between cage and extended 5' leaders}

\item{filteredCage}{The filtered raw cage-data
used to reassign 5' leaders}
}
\value{
a data.table of max peaks
}
\description{
Find max peak for each transcript,
returns as data.table, without names, but with index
}
\keyword{internal}
